use strict;
use warnings;

# this test was generated with Dist::Zilla::Plugin::Test::EOL 0.19

use Test::More 0.88;
use Test::EOL;

my @files = (
    'Changes',
    'GPLv3',
    'VERSION',
    'ex/AdaptiveTestVersion/README.pod',
    'ex/AdaptiveTestVersion/dist.ini',
    'ex/Description/README.pod',
    'ex/Description/dist.ini',
    'ex/TemplateVariables/BUGS.pod',
    'ex/TemplateVariables/README.pod',
    'ex/TemplateVariables/dist.ini',
    'ex/UnwantedDependencies/README.pod',
    'ex/UnwantedDependencies/dist.ini',
    'ex/VersionHandling/README.pod',
    'ex/VersionHandling/VERSION',
    'ex/VersionHandling/dist.ini',
    'lib/Dist/Zilla/Plugin/Hook.pm',
    'lib/Dist/Zilla/Plugin/Hook/Manual.pod',
    'lib/Dist/Zilla/Role/Hooker.pm',
    't/hook.t',
    't/lib/HookTester.pm',
    'xt/aspell-en.pws',
    'xt/examples.t',
    'xt/perlcritic.ini'
);

eol_unix_ok($_, { trailing_whitespace => 1 }) foreach @files;
done_testing;
