use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.007003
use Test::Spelling 0.12;
use Pod::Wordlist;

set_spell_cmd('aspell list -l en -p ./xt/aspell-en.pws');
add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib ) );
__DATA__
AfterBuild
AfterMint
AfterRelease
BeforeArchive
BeforeBuild
BeforeMint
BeforeRelease
Bugger
Dist
FileGatherer
FileMunger
FilePruner
Hook
Hooker
Init
InstallTool
LicenseProvider
Manual
MetaProvider
ModuleMaker
NameProvider
Plugin
PrereqSource
ReleaseStatusProvider
Releaser
Role
Van
VersionProvider
Zilla
de
lib
van
